import { compact } from '../../util/array.js';
import { _load } from '../../util/loader.js';
import { timerify } from '../../util/performance.js';
export const NAME = 'Webpack';
export const ENABLERS = ['webpack'];
export const isEnabled = ({ dependencies }) => ENABLERS.some(enabler => dependencies.has(enabler));
export const CONFIG_FILE_PATTERNS = ['webpack.config*.{js,ts}'];
const resolveRuleSetLoaders = (rule) => {
    if (!rule || typeof rule === 'string')
        return [];
    if (typeof rule.use === 'string')
        return [rule.use];
    if (Array.isArray(rule.use))
        return rule.use.flatMap(resolveUseItemLoader);
    return rule.use && typeof rule.use !== 'function' ? resolveUseItemLoader(rule.use) : [];
};
const resolveUseItemLoader = (use) => {
    if (!use)
        return [];
    if (typeof use === 'string')
        return [use];
    if ('loader' in use && typeof use.loader === 'string')
        return [use.loader];
    return [];
};
const findWebpackDependencies = async (configFilePath, { manifest }) => {
    let config = await _load(configFilePath);
    if (typeof config === 'function') {
        config = config();
    }
    const loaders = (config.module?.rules?.flatMap(resolveRuleSetLoaders) ?? [])
        .map(loader => loader.replace(/\?.*/, ''))
        .filter(loader => !loader.startsWith('/'));
    const scripts = Object.values(manifest.scripts ?? {});
    const webpackCLI = scripts.some(script => script?.includes('webpack ')) ? ['webpack-cli'] : [];
    const webpackDevServer = scripts.some(script => script?.includes('webpack serve')) ? ['webpack-dev-server'] : [];
    return compact([...loaders, ...webpackCLI, ...webpackDevServer]);
};
export const findDependencies = timerify(findWebpackDependencies);
